{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  dev_module.pas : C++/Pascal IDE module            *
*                                                    *
******************************************************
}

unit dev_module;

interface
uses Libc, SysUtils, IniFiles, QDialogs, QGraphics, infofrm, project,
  {$IFDEF CPP}
  cpp_compoptions, SynHighlighterCpp,
  {$ELSE}
  pas_compoptions, SynHighlighterPas,
  {$ENDIF}
  editor, version, utils;

type
  TDevModule = class(TObject)

  constructor Create;
  procedure Free;
  private

  public
    Name : string;
    Filter : string;
    ToolsIniFileName : string;
    IniFileName : string;
    Compiler : string;
    ConfDir : string;
    TemplateDir : string;
    MainCode : string;
    Make : string;
    Term : string;
    Gdb : string;
    CompileParams : string;
    UserParams : string;
    CompilerOptionsModified : boolean;
    {$IFDEF CPP}
    Syn : TSynCppSyn;
    CppCompiler : string;
    CppCompileParams : string;
    CompOptForm : TCpp_CompForm;
    {$ELSE}
    Syn : TSynPasSyn;
    CompOptForm : TPas_CompForm;
    {$ENDIF}

    procedure NewConsoleFile(Editor : TEditor);
    procedure CreateMakefile;
    procedure CreateStaticMakefile;
    procedure CreateDynamicMakefile;
    procedure AnalyseOutput(BinStr : string);
    procedure GetCompileParams;
    procedure LoadSyntaxAttributes;
    procedure CreateNewCompletionFile;
  end;

var
  Module : TDevModule;

implementation
uses main, outputfrm;

constructor TDevModule.Create;
begin
  inherited Create;
  {$IFDEF CPP}
  Name := 'Dev-C++';
  Filter := 'C/C++ sources and headers (*.c;*.cpp;*.h;*.hpp)|C++ source file (*.cpp)|C source file (*.c)|Header file (*.h)|C++ header file (*.hpp)|All files (*.*)';
  ConfDir := GetEnvironmentVariable('HOME') + '/.devcpp';
  IniFileName := ConfDir + '/config';
  MainForm.IniFile := TIniFile.Create(IniFileName);
  MainForm.IniFile.CaseSensitive := false;
  Compiler := MainForm.IniFile.ReadString('dev','gcc_dir', GCC);
  CppCompiler := MainForm.IniFile.ReadString('dev','g++_dir', GPP);
  MainCode := 'int main(int argc, char **argv)'+#10+
              '{'+#10+
              #10+
              '  return (0);'+#10+
              '}'+#10;
  Syn := TSynCppSyn.Create(MainForm);
  {$ELSE}
  Name := 'Dev-Pascal';
  Filter := 'Pascal source file (*.pas)|Free Pascal source file (*.pp)|All files (*.*)';
  ConfDir := GetEnvironmentVariable('HOME') + '/.devpas';
  IniFileName := ConfDir + '/config';
  MainForm.IniFile := TIniFile.Create(IniFileName);
  MainForm.IniFile.CaseSensitive := false;
  Compiler := MainForm.IniFile.ReadString('dev','ppc_dir', PPC386);
  MainCode := 'program '+UNTITLED+';'+#10+
              #10+
              'begin'+
              #10+
              'end.'+#10;
  Syn := TSynPasSyn.Create(MainForm);
  {$ENDIF}
  LoadSyntaxAttributes;
  Make := MainForm.IniFile.ReadString('dev', 'make_dir', MAKE);
  Term := MainForm.IniFile.ReadString('dev', 'term_dir', XTERM);
  Gdb := MainForm.IniFile.ReadString('dev', 'gdb_dir', GDB);
  ToolsIniFileName := ConfDir + '/tools_config';
  TemplateDir := ConfDir + '/templates';
  if (Make = '') or (Compiler = '') {$IFDEF CPP} or (CppCompiler = '') {$ENDIF} then
    MessageDlg('Your compiler and make settings are not configured, please go to Compiler Options dialog', mtWarning, [mbOK], 0);
  if (Term = '') then
    MessageDlg('Your xterm, aterm or Eterm path has not been configured, please go to Environment Options dialog', mtWarning, [mbOK], 0);
  if not DirectoryExists(ConfDir) then
    mkdir(ConfDir);
  if not DirectoryExists(TemplateDir) then
    mkdir(TemplateDir);
end;

procedure TDevModule.Free;
begin
  inherited Free;
end;

procedure TDevModule.LoadSyntaxAttributes;
begin
  with MainForm.IniFile do
  with Syn do begin
    CommentAttri.Background := clWhite;
    CommentAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','CommentAttri',''), 0, clBlack);
                           GetStyles(CommentAttri, 'CommentAttri');
    IdentifierAttri.Background := clWhite;
    IdentifierAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','IdentifierAttri',''), 0, clBlack);
                           GetStyles(IdentifierAttri, 'IdentifierAttri');
    KeyAttri.Background := clWhite;
    KeyAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','KeyAttri',''), 0, clBlack);
                           GetStyles(KeyAttri, 'KeyAttri');
    NumberAttri.Background := clWhite;
    NumberAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','NumberAttri',''), 0, clBlack);
                           GetStyles(NumberAttri, 'NumberAttri');
    SpaceAttri.Background := clWhite;
    SpaceAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','SpaceAttri',''), 0, clBlack);
                           GetStyles(SpaceAttri, 'SpaceAttri');
    StringAttri.Background := clWhite;
    StringAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','StringAttri',''), 0, clBlack);
                           GetStyles(StringAttri, 'StringAttri');
    SymbolAttri.Background := clWhite;
    SymbolAttri.Foreground :=
      MakeCommaTextToColor(ReadString('EditorColor','SymbolAttri',''), 0, clBlack);
                           GetStyles(SymbolAttri, 'SymbolAttri');
    DirecAttri.Background  := clWhite;
    DirecAttri.Foreground  := MakeCommaTextToColor(ReadString('EditorColor','DirecAttri',''), 0, clGreen);
                              GetStyles(DirecAttri, 'DirecAttri');
    InvalidAttri.Background := clWhite;
  end;
end;

procedure TDevModule.NewConsoleFile(Editor : TEditor);
begin
  {$IFDEF CPP}
   Editor.Text.Lines.Add('');
   Editor.Text.Lines.Add('int main(int argc, char **argv)');
   Editor.Text.Lines.Add('{');
   Editor.Text.Lines.Add('');
   Editor.Text.Lines.Add('  return(0);');
   Editor.Text.Lines.Add('}');
  {$ELSE}
   Editor.Text.Lines.Add('program '+UNTITLED+';');
   Editor.Text.Lines.Add('');
   Editor.Text.Lines.Add('begin');
   Editor.Text.Lines.Add('');
   Editor.Text.Lines.Add('end.');
  {$ENDIF}
end;

procedure write_make_clean(var F : TextFile);
begin
  writeln(F);
  writeln(F,'clean :');
  writeln(F, #9 + 'rm -f $(OBJ) $(BIN)');
end;

procedure TDevModule.CreateMakefile;
var F : TextFile;
    SrcStr, ObjStr, CcStr : string;
    {$IFDEF CPP}
    i : integer;
    {$ENDIF}
begin
  with MainForm do begin
  {$IFDEF CPP}
  for  i := 0 to Project.UnitCount - 1 do begin
    if (ExtractFileExt(Project.Units[i].FileName) = '.h') or
       (ExtractFileExt(Project.Units[i].FileName) = '.hpp') then
        next;
    SrcStr := SrcStr + Project.Units[i].FileName + ' ';
    ObjStr := ObjStr + ChangeFileExt(Project.Units[i].FileName, '.o') + ' ';
  end;
  if Project.UseGpp then
    CCStr := 'g++'
  else
    CCStr := 'cc';
  {$ELSE}
  SrcStr := Project.Mainunit;
  ObjStr := '';
  CCStr := 'ppc386';
  {$ENDIF}
  Assignfile(F, ExtractFilePath(Project.FileName) + DEV_MAKEFILE);
  Rewrite(F);
  writeln(F,'# Project : ' + Project.Name);
  writeln(F,'# Makefile created by ' + Module.Name + ' ' + DEV_VERSION);
  writeln(F);
  writeln(F,'CC  = '+CCstr);
  writeln(F,'SRC = '+SrcStr);
  writeln(F,'OBJ = '+ObjStr);
  writeln(F,'BIN = '+Project.GetExecutableName);
  GetCompileParams;
  {$IFDEF CPP}
  if (Project.UseGpp) then
    writeln(F, 'CFLAGS = '+ CppCompileParams)
  else
    writeln(F, 'CFLAGS = '+ CompileParams);
  writeln(F);
  writeln(F,'all : $(OBJ)');
  writeln(F, #9 + '$(CC) $(OBJ) -o $(BIN)');
  {$ELSE}
  writeln(F, 'FLAGS = -r '+ CompileParams);
  writeln(F);
  writeln(F,'all : ');
  writeln(F, #9 + '$(CC) $(SRC) -o$(BIN) $(FLAGS)');
  {$ENDIF}
  write_make_clean(F);
  CloseFile(F);
  end;
end;

procedure TDevModule.CreateStaticMakefile;
var F : TextFile;
    SrcStr, ObjStr, CcStr : string;
    {$IFDEF CPP}
    i : integer;
    {$ENDIF}
begin
  with MainForm do begin
  {$IFDEF CPP}
  for  i := 0 to Project.UnitCount - 1 do begin
    if (ExtractFileExt(Project.Units[i].FileName) = '.h') or
       (ExtractFileExt(Project.Units[i].FileName) = '.hpp') then
        next;
    SrcStr := SrcStr + Project.Units[i].FileName + ' ';
    ObjStr := ObjStr + ChangeFileExt(Project.Units[i].FileName, '.o') + ' ';
  end;
  if Project.UseGpp then
    CCStr := 'g++'
  else
    CCStr := 'cc';
  {$ELSE}
  SrcStr := Project.Mainunit;
  ObjStr := '';
  CCStr := 'ppc386';
  {$ENDIF}
  Assignfile(F, ExtractFilePath(Project.FileName) + DEV_MAKEFILE);
  Rewrite(F);
  writeln(F,'# Project : ' + Project.Name);
  writeln(F,'# Makefile created by ' + Module.Name + ' ' + DEV_VERSION);
  writeln(F);
  writeln(F,'CC  = '+CCstr);
  writeln(F,'SRC = '+SrcStr);
  writeln(F,'OBJ = '+ObjStr);
  writeln(F,'BIN = '+Project.GetExecutableName);
  writeln(F);
  {$IFDEF CPP}
  if (Project.UseGpp) then
    writeln(F, 'CFLAGS = '+ CppCompileParams)
  else
    writeln(F, 'CFLAGS = '+ CompileParams);
  writeln(F);
  writeln(F, 'all : $(OBJ)');
  writeln(F, #9 + 'ar r "$(BIN)" $(OBJ)');
  writeln(F, #9 + 'ranlib "$(BIN)"');
  {$ELSE}
  writeln(F, 'FLAGS = -r '+ CompileParams);
  writeln(F);
  writeln(F, 'all : ');
  writeln(F, #9 + '$(CC) $(SRC) -CS -o$(BIN) $(FLAGS)');
  {$ENDIF}
  write_make_clean(F);
  CloseFile(F);
  end;
end;

procedure TDevModule.CreateDynamicMakefile;
var F : TextFile;
    SrcStr, ObjStr, CcStr : string;
    {$IFDEF CPP}
    i : integer;
    {$ENDIF}
begin
  with MainForm do begin
  {$IFDEF CPP}
  for  i := 0 to Project.UnitCount - 1 do begin
    if (ExtractFileExt(Project.Units[i].FileName) = '.h') or
       (ExtractFileExt(Project.Units[i].FileName) = '.hpp') then
        next;
    SrcStr := SrcStr + Project.Units[i].FileName + ' ';
    ObjStr := ObjStr + ChangeFileExt(Project.Units[i].FileName, '.o') + ' ';
  end;
  if Project.UseGpp then
    CCStr := 'g++'
  else
    CCStr := 'cc';
  {$ELSE}
  SrcStr := Project.Mainunit;
  ObjStr := '';
  CCStr := 'ppc386';
  {$ENDIF}
  Assignfile(F, ExtractFilePath(Project.FileName) + DEV_MAKEFILE);
  Rewrite(F);
  writeln(F,'# Project : ' + Project.Name);
  writeln(F,'# Makefile created by ' + Module.Name + ' ' + DEV_VERSION);
  writeln(F);
  writeln(F,'CC  = '+CCstr);
  writeln(F,'SRC = '+SrcStr);
  writeln(F,'OBJ = '+ObjStr);
  writeln(F,'BIN = '+Project.GetExecutableName);
  writeln(F,'NAMEBIN = '+ExtractFileName(Project.GetExecutableName));
  writeln(F);
  {$IFDEF CPP}
  if (Project.UseGpp) then
    writeln(F, 'CFLAGS = '+ CppCompileParams)
  else
    writeln(F, 'CFLAGS = '+ CompileParams);
  writeln(F);
  writeln(F,'all : $(OBJ)');
  writeln(F, #9 + 'ld -o $(BIN) $(OBJ) -soname lib$(NAMEBIN)');
  {$ELSE}
  writeln(F, 'FLAGS = -r '+ CompileParams);
  writeln(F);
  writeln(F, 'all : ');
  writeln(F, #9 + '$(CC) $(SRC) -CD -o$(BIN) $(FLAGS)');
  {$ENDIF}
  write_make_clean(F);
  CloseFile(F);
  end;
end;

{$IFDEF CPP}
procedure TDevModule.AnalyseOutput(BinStr : string);
var F, R : TextFile;
    c : char;
    MessageStr, LineStr, UnitStr, tmp : string;
    TotalError : integer;
label Start, CompileError, TestAgain;
const MAXERROR = 35;

procedure Getch;
begin
  read(F, c);
  tmp := tmp + c ;
end;

// Delete line break from error message
function CureStr(s : string) : string;
begin
  while pos(#10, s)<>0 do
    Delete(s,pos(#10, s),1);
  Result := s;
end;

begin
  AssignFile(F, Module.ConfDir + DEV_COMPILELOG);
  Reset(F);
  TotalError := 0;
  OutputForm.Output.Lines.LoadFromFile(Module.ConfDir + DEV_COMPILELOG);
  if EOF(F) and FileExists(BinStr) then begin
    InfoForm.OkBtn.Enabled := True;
    InfoForm.ShowResultsBtn.Enabled := False;
    InfoForm.ExecBtn.Enabled := True;
    InfoForm.MoreBtn.Enabled := True;
    InfoForm.TotalErrors.Caption := '0';
    AssignFile(R, BinStr);
    Reset(R);
    InfoForm.SizeFile.Caption := IntToStr(FileSize(R));
    CloseFile(R);
    CloseFile(F);
    exit;
  end
  else begin
    // reads the compiler message files for errors and warnings and interprets it
    Start:
    MessageStr := '';
    LineStr := '';
    UnitStr := '';
    tmp := '';
    c := 'A';   // initiate analyser
    getch;
    while (not EOF(F)) and (c <> '') do begin
      if c = 'G' then begin
        repeat
          Getch;
        until c = ':';
        Getch;
        repeat
          MessageStr := MessageStr + c;
          Getch;
        until c = #10;
        MainForm.AddItem('','Compiler',CureStr(MessageStr));
        inc(TotalError);
        goto Start;
      end;
      UnitStr := '';
      repeat
        if c <> ':' then
          UnitStr := UnitStr + c;
        Getch;
      until (c = ':') or (EOF(F));
      LineStr := '';
      getch;
      while (c in ['0'..'9']) and (not EOF(F)) do begin
        LineStr := LineStr + c;
        Getch;
      end;
      Getch;
      MessageStr := c;
      if c = 'I' then begin
        TestAgain :
        repeat
          getch;
          write(c);
       {   if c = ',' then begin
            repeat
              Getch
            until (c = #10) or EOF(F);
            goto TestAgain;
          end;                        }
        until (c = #10) or EOF(F);
        goto Start;
      end;
      Getch;
      if c = ':' then
        goto Start
      else begin
        repeat
          MessageStr := MessageStr + c;
          Getch;
        until (c = #10) or EOF(F);
      end;
      Getch;
      if c = 'I' then
        goto TestAgain;
      while Pos('  ', MessageStr) <> 0 do
        Delete(MessageStr, Pos('  ', MessageStr), 1);
      if (pos('***', MessageStr) <> 0) then begin
        MainForm.MakeOutput.Items.Append(MessageStr);
        MainForm.MakeSheet.Highlighted := True;
      end
      else if (MessageStr <> '') then
        MainForm.AddItem(' '+LineStr,lowercase(UnitStr),CureStr(MessageStr));
      if pos('warning:', MessageStr) <> 0 then
        inc(TotalError);
    end
  end;
  if (not EOF(F)) and (TotalError < MAXERROR) then
    goto Start;
  CloseFile(F);
  InfoForm.OkBtn.Enabled := True;
  InfoForm.ShowResultsBtn.Enabled := True;
  InfoForm.ExecBtn.Enabled := False;
  InfoForm.MoreBtn.Enabled := False;
  InfoForm.TotalErrors.Caption := IntToStr(TotalError);
  InfoForm.SizeFile.Caption := '0';
  if (MainForm.MessageControl.ActivePage <> MainForm.CompSheet) and (Mainform.CompilerOutput.Items.Count > 0) then
     MainForm.CompSheet.Highlighted := True;
end;

procedure TDevModule.GetCompileParams;
begin
  { Check user's compiler options }
  with MainForm do begin
    CompileParams := '';
    CppCompileParams := '';
    UserParams := '';

    if ProjectOpen then
      UserParams := Project.CompilerOptions;
  if IniFile.ReadBool('Directories','AddCommands',True) then
      UserParams := UserParams+' '+ IniFile.ReadString('Directories','Commands',UserParams);
  if IniFile.ReadBool('C_Compiler','Ansi',False) then
     CompileParams := '-ansi ';
  if IniFile.ReadBool('C_Compiler','Traditional',False) then
     CompileParams := CompileParams + '-traditional-cpp ';
  if IniFile.ReadBool('C_Compiler','Warning',False) then
     CompileParams := CompileParams + '-w ';
  if IniFile.ReadBool('C++_Compiler','AccessChecking',False) then
     CppCompileParams := CppCompileParams + '-fno-access-control ';
  if IniFile.ReadBool('C++_Compiler','AcceptDollar',False) then
     CppCompileParams := CppCompileParams + '-fdollars-in-identifiers ';
  if IniFile.ReadBool('C++_Compiler','Heuristics',False) then
     CppCompileParams := CppCompileParams + '-fsave-memoized ';
  if IniFile.ReadBool('Code_Generation','Exceptions',False) then  begin
     CompileParams := CompileParams + '-fexceptions ';
     CppCompileParams := CppCompileParams + '-fexceptions ';
  end;
  if IniFile.ReadBool('Code_Generation','DoubleFloat',False) then begin
     CompileParams := CompileParams + '-fshort-double ';
     CppCompileParams := CppCompileParams + '-fshort-double ';
  end;
  if IniFile.ReadBool('Code_Generation','Memory',False) then begin
     CompileParams := CompileParams + '-fverbose-asm ';
     CppCompileParams := CppCompileParams + '-fverbose-asm ';
     end;
  if IniFile.ReadBool('Linker','ObjC',False) then begin
     CompileParams := CompileParams + '-lobjc ';
     CppCompileParams := CppCompileParams + '-lobjc ';
  end;
  if IniFile.ReadBool('Linker','DebugInfo',False) then begin
     CompileParams := CompileParams + '-ggdb ';
     CppCompileParams := CppCompileParams + '-ggdb ';
  end
  else begin
     CompileParams := CompileParams + '-s ';
     CppCompileParams := CppCompileParams + '-s ';
  end;
  if IniFile.ReadBool('Linker','NoLibs',False) then begin
     CompileParams := CompileParams + '-nostdlib ';
     CppCompileParams := CppCompileParams + '-nostdlib ';
  end;
  if IniFile.ReadBool('Optimization','Optimize',False) then begin
     CompileParams := CompileParams + '-O1 ';
     CppCompileParams := CppCompileParams + '-O1 ';
  end;
  if IniFile.ReadBool('Optimization','Best',False) then begin
     CompileParams := CompileParams + '-O3 ';
     CppCompileParams := CppCompileParams + '-O3 ';
  end;
  if IniFile.ReadBool('Optimization','MinorOpt',False) then begin
     CompileParams := CompileParams + '-fexpensive-optimizations ';
     CppCompileParams := CppCompileParams + '-fexpensive-optimizations ';
  end;
  end
end;
{$ENDIF}

{$IFDEF PAS}
procedure TDevModule.AnalyseOutput(BinStr : string);
var c : char;
    MessageStr, LineStr, UnitStr, tmp : string;
    TotalError : integer;
    F : TextFile;
    R : File;
label Start, GoOut, CompileError, TestAgain, FatalError;
const MaxError = 35;

procedure Getch;
begin
  read(F, c);
  tmp := tmp + c ;
end;

// Delete line break from error message
function CureStr(s : string) : string;
begin
  while pos(#10, s)<>0 do
    Delete(s,pos(#10, s),1);
  Result := s;
end;

begin
  AssignFile(F, Module.ConfDir + DEV_COMPILELOG);
  OutputForm.Output.Lines.LoadFromFile(Module.ConfDir + DEV_COMPILELOG);
  Reset(F);
  TotalError := 0;
  if EOF(F) and FileExists(BinStr) then begin
    InfoForm.OkBtn.Enabled := True;
    InfoForm.ShowResultsBtn.Enabled := False;
    InfoForm.ExecBtn.Enabled := True;
    InfoForm.MoreBtn.Enabled := True;
    InfoForm.TotalErrors.Caption := '0';
    AssignFile(R, BinStr);
    Reset(R);
    InfoForm.SizeFile.Caption := IntToStr(FileSize(R));
    CloseFile(R);
    CloseFile(F);
    exit;
  end
  else begin
    Start :
    MessageStr := '';
    LineStr := '';
    UnitStr := '';
    if MainForm.Project <> nil then
      readln(F);
    readln(F, tmp);
    if (pos('Error:', tmp) <> 0) or (pos('Fatal', tmp) <> 0) then begin
       MainForm.additem('', 'Compiler', tmp);
       goto GoOut;
    end;
    readln(F);
    readln(F);
    readln(F);
    c := 'A';   // initiate analyser
    while (not EOF(F)) and (c <> '') do begin
      if (TotalError > MaxError) then
        goto GoOut;
      UnitStr := '';
      Getch;
      write(c);
      repeat
        if c <> '(' then
           UnitStr := UnitStr + c;
        getch;
      until (c = '(') or (c = '') or (c = ':') or EOF(F);
      write(c);
      getch;
      if UnitStr = 'Fatal' then begin
        UnitStr := 'Fatal Error';
        goto FatalError;
      end;
      LineStr := '';
      while c in ['0'..'9'] do begin
        LineStr := LineStr + c;
        getch;
      end;
      if c = ',' then begin
         LineStr := LineStr+' / ';
         getch;
         while c in ['0'..'9'] do begin
           LineStr := LineStr + c;
           getch;
         end;
      end;
      FatalError :
      getch;
      MessageStr := c;
      getch;
      repeat
        MessageStr := MessageStr + c;
        getch;
        printf(pchar('huhu : %d'), c);
      until (c = #13) or (c = '')or EOF(F);
      writeln(MessageStr);
      Getch;
      while Pos('  ', MessageStr)<>0 do
        Delete(MessageStr, Pos('  ', MessageStr), 1);
      MainForm.AddItem(LineStr, UnitStr, CureStr(MessageStr));
      if (pos('Warning:', MessageStr)=0) and (pos('Hint:', MessageStr)=0)
        and (pos('Note:', MessageStr)=0) then
        inc(TotalError);
      if EOF(F) then goto GoOut
      else goto Start;
    end;
    GoOut:
    CloseFile(F);
    InfoForm.OkBtn.Enabled := True;
    InfoForm.ShowResultsBtn.Enabled := True;
    InfoForm.ExecBtn.Enabled := False;
    InfoForm.MoreBtn.Enabled := False;
    InfoForm.TotalErrors.Caption := IntToStr(TotalError);
    InfoForm.SizeFile.Caption := '0';
    InfoForm.TotalErrors.Caption := IntToStr(TotalError);
    if (MainForm.MessageControl.ActivePage <> MainForm.CompSheet) and (MainForm.CompilerOutput.Items.Count > 0) then
      MainForm.CompSheet.Highlighted := True;
end
end;

procedure TDevModule.GetCompileParams;
begin
  { Check user's compiler options }
  if not CompilerOptionsModified then exit;
  with MainForm do begin
  CompileParams := '';
  UserParams := '';

  if ProjectOpen then
     UserParams := Project.CompilerOptions;
  if IniFile.ReadBool('Directories','AddCommands',True) then
      UserParams := UserParams+' '+ IniFile.ReadString('Directories','Commands',UserParams);
  if IniFile.ReadBool('Pascal_Compiler','DelphiExt',True) then
     CompileParams := '-S2 ';
  if IniFile.ReadBool('Pascal_Compiler','Assert',False) then
     CompileParams := CompileParams + '-Sa ';
  if IniFile.ReadBool('Pascal_Compiler','Ansi',False) then
     CompileParams := CompileParams + '-Sh ';
  if IniFile.ReadBool('Pascal_Compiler','TP7',True) then
     CompileParams := CompileParams + '-So ';
  if IniFile.ReadBool('Pascal_Compiler','COp',False) then
     CompileParams := CompileParams + '-Sc ';
  if IniFile.ReadBool('Pascal_Compiler','LabelGoto',True) then
     CompileParams := CompileParams + '-Sg ';
  if IniFile.ReadBool('Pascal_Compiler','DelphiComp',True) then
     CompileParams := CompileParams + '-Sd ';
  if IniFile.ReadBool('Pascal_Compiler','CInline',False) then
     CompileParams := CompileParams + '-Si ';
  if IniFile.ReadBool('Pascal_Compiler','Macros',False) then
     CompileParams := CompileParams + '-Sm ';
  if IniFile.ReadBool('Pascal_Compiler','LinkDyn',False) then
     CompileParams := CompileParams + '-XD ';
  if IniFile.ReadBool('Pascal_Compiler','LinkSmart',False) then
     CompileParams := CompileParams + '-XX ';
  if IniFile.ReadBool('Pascal_Compiler','StripSymbols',False) then
     CompileParams := CompileParams + '-Xs ';
  if IniFile.ReadBool('Pascal_Compiler','NoCheckUnit',True) then
     CompileParams := CompileParams + '-Un ';
  if IniFile.ReadBool('Code_Generation','IOCheck',False) then
     CompileParams := CompileParams + '-Ci ';
  if IniFile.ReadBool('Code_Generation','StackCheck',False) then
     CompileParams := CompileParams + '-Ct ';
  if IniFile.ReadBool('Code_Generation','Overflow',False) then
     CompileParams := CompileParams + '-Co ';
  if IniFile.ReadBool('Optimization','SmallCode',False) then
     CompileParams := CompileParams + '-Og ';
  if IniFile.ReadBool('Optimization','QuickOpt',True) then
     CompileParams := CompileParams + '-O1 ';
  if IniFile.ReadBool('Optimization','Best',False) then
     CompileParams := CompileParams + '-O2 ';
  if IniFile.ReadBool('Linker','DebugInfo',False) then
     CompileParams := CompileParams + '-g ';
  if IniFile.ReadBool('Linker','NoReloc',False) then
     CompileParams := CompileParams + '-WN ';
  end;
end;

{$ENDIF}

procedure TDevModule.CreateNewCompletionFile;
var F : TextFile;
begin
  try
    AssignFile(F, ConfDir + DEV_COMPLETION_FILE);
    Rewrite(F);
    {$IFDEF CPP}
    writeln(F, 'int main(int argc, char **argv)');
    writeln(F, 'for (; ; )');
    {$ELSE}
    writeln(F, 'for  :=  to ');
    {$ENDIF}
    CloseFile(F);
  except
    MessageDlg('Could not create Code Completion list file.', mtError, [mbOK], 0);
  end;
end;

end.
