{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  project.pas : project management object           *
*                                                    *
******************************************************
}

unit project;

interface
uses IniFiles, SysUtils, QDialogs, QComCtrls, Editor,
     Classes, QControls, version;

var ProjectOpen : Boolean;

const T_Console = 0;
const T_Gui = 1;
const T_Static = 2;
const T_Dynamic = 3;


type
     TProjUnit = record
                  Editor : TEditor;
                  FileName : string;
                  New : boolean;
                  Node     : TTreeNode;
                 end;
     TProjUnits = array of TProjUnit;

     TProject = class(TObject)
     constructor Create(nFileName, nName : string);
     procedure Free;
     procedure NewUnit(NewProject : boolean);
     procedure AddUnit(s : string);
     procedure OpenUnit(index : integer);
     procedure SaveUnitAs(i : integer; sFileName : string);
     procedure Save;
     procedure SaveUnitCount;
     procedure Write(name, value : string);
     procedure WriteBool(name : string; value : boolean);
     function Read(name, defaultvalue : string) : string;
     function ReadBool(name : string; defaultvalue : boolean) : boolean;
     function MakeProjectNode : TTreeNode;
     function MakeNewFileNode(s : string) : TTreeNode;
     procedure Update;
     procedure UpdateFile;
     procedure Open;
     function FileAlreadyExists(s : string) : boolean;
     procedure Remove(index : integer);
     function GetUnitFromEditor(ed : TEditor) : integer;
     function GetExecutableName : string;

     public
       Name      : string;
       NoConsole : boolean;
       StaticLib : boolean;
       DynamicLib : boolean;
       FileName  : string;
       UnitCount : integer;
       ObjFiles   : string;
       CompilerOptions : string;
       IncludeDirs : string;
       Libs : string;
       ProjectType : integer;
       Node : TTreeNode;
       IniFile : TIniFile;
       {$IFDEF CPP}
       UseGpp : boolean;
       {$ELSE}
       MainUnit : string;
       {$ENDIF}
       Units : TProjUnits;
     private

     end;

implementation
uses dev_module, main;

constructor TProject.Create(nFileName, nName: string);
begin
  inherited Create;
  ProjectOpen := True;
  FileName := nFileName;
  IniFile := TIniFile.Create(nFileName);
  if nName = '$__DEVINTERNAL_OPEN' then
    Open
  else begin
    Name := nName;
    IniFile.Clear;
    IniFile.WriteString('project','filename',nFileName);
    IniFile.WriteString('project','name',nName);
    UnitCount := 0;
    Node := MakeProjectNode;
  end;
end;

procedure TProject.Free;
var i : integer;
begin
  for i := 0 to unitCount - 1 do
    Units[i].Node.Free;
  IniFile.Free;
  Node.Free;
  ProjectOpen := False;
  inherited Free;
end;

function TProject.MakeProjectNode : TTreeNode;
begin
  MakeProjectNode := MainForm.ProjectView.Items.Add(nil, Name);
  MakeProjectNode.SelectedIndex:=0;
  MakeProjectNode.ImageIndex:=0;
  MainForm.ProjectView.FullExpand;
end;

function TProject.MakeNewFileNode(s : string) : TTreeNode;
begin
  MakeNewFileNode := MainForm.ProjectView.Items.AddChild(Node, s);
  MakeNewFileNode.SelectedIndex := 1;
  MakeNewFileNode.ImageIndex := 1;
  MainForm.ProjectView.FullExpand;
end;

procedure TProject.NewUnit(NewProject : boolean);
begin
  inc(UnitCount);
  SaveUnitCount;
  SetLength(Units, UnitCount);
  Units[UnitCount - 1].Filename := UNTITLED + IntToStr(Unitcount);
  Units[UnitCount - 1].New := True;
  Units[UnitCount - 1].Editor := TEditor.Create(MainForm);
  Units[UnitCount - 1].Editor.Init(true, Units[UnitCount - 1].Filename, '', false);
  Units[UnitCount - 1].Node := MakeNewFileNode(ExtractFileName(Units[UnitCount - 1].FileName));
  if NewProject then
    Module.NewConsoleFile(Units[UnitCount - 1].Editor);
  IniFile.WriteString('unit'+IntToStr(UnitCount), 'filename', Units[UnitCount - 1].Filename);
  {$IFDEF PAS}
  if UnitCount = 1 then
    MainUnit := Units[0].Filename;
  {$ENDIF}
  IniFile.UpdateFile;
end;

procedure TProject.AddUnit(s : string);
begin
  inc(UnitCount);
  SetLength(Units, UnitCount);
  Units[UnitCount - 1].Filename := s;
  Units[UnitCount - 1].New := false;
  Units[UnitCount - 1].Editor := TEditor.Create(MainForm);
  Units[UnitCount - 1].Editor.Init(true, ExtractFilename(Units[UnitCount - 1].Filename), Units[UnitCount - 1].Filename, true);
  Units[UnitCount - 1].Node := MakeNewFileNode(ExtractFileName(Units[UnitCount - 1].FileName));
  IniFile.WriteString('unit'+IntToStr(UnitCount), 'filename', Units[UnitCount - 1].Filename);
  SaveUnitCount;
  Save;
end;

procedure TProject.Update;
begin
  Name := Read('name', '');
  Objfiles := Read('objfile', '');
  Libs := Read('libs', '');
  IncludeDirs := Read('includedirs', '');
  Compileroptions := Read('compileroptions', '');
  NoConsole := ReadBool('noconsole', false);
  StaticLib := ReadBool('static', false);
  DynamicLib := ReadBool('dynamic', false);
  {$IFDEF CPP}
  UseGpp := ReadBool('usegpp', False);
  {$ELSE}
  MainUnit := Read('mainunit', '');
  {$ENDIF}
end;

procedure TProject.UpdateFile;
begin
  Write('name', Name);
  Write('objfile', Objfiles);
  Write('libs', Libs);
  Write('includedirs', IncludeDirs);
  Write('compileroptions', CompilerOptions);
  WriteBool('noconsole', NoConsole);
  WriteBool('static', StaticLib);
  WriteBool('dynamic', DynamicLib);
  {$IFDEF CPP}
  WriteBool('usegpp',UseGpp);
  {$ELSE}
  Write('mainunit', MainUnit);
  {$ENDIF}
  Save;
end;

procedure TProject.Open;
var i : integer;
begin
  Update;
  UnitCount := IniFile.ReadInteger('project','unitcount', 0);
  SetLength(Units, UnitCount);
  Node := MakeProjectNode;
  for i := 0 to UnitCount - 1 do begin
    Units[i].Filename := IniFile.Readstring('unit'+IntToStr(i + 1), 'filename', '');
    Units[i].New := False;
    Units[i].Node := MakeNewFileNode(ExtractFileName(Units[i].FileName));
  end;
  if MainForm.IniFile.ReadBool('Options', 'OpenAll', False) then
    for i := 0 to UnitCount - 1 do
      OpenUnit(i)
  else if (UnitCount > 0) then
    OpenUnit(0);
end;

procedure TProject.Save;
begin
  IniFile.UpdateFile;
end;

procedure TProject.SaveUnitCount;
begin
  IniFile.WriteInteger('project','unitcount', UnitCount);
  Save;
end;

procedure TProject.Remove(index : integer);
var i : integer;
begin
  if Units[index].Editor <> nil then
    Units[index].Editor.Destroy;
  MainForm.ProjectView.Items.Delete(Units[Index].Node);
  for i := Index to UnitCount - 2 do begin
    Units[i] := Units[i+1];
    IniFile.WriteString('unit'+IntToStr(i+1),'filename', Units[i].FileName);
  end;
  IniFile.EraseSection('unit'+IntToStr(UnitCount));
  Dec(UnitCount);
  SaveUnitCount;
  SetLength(Units, UnitCount);
  IniFile.EraseSection('Unit'+IntToStr(UnitCount+1));
  Save;
end;

function TProject.FileAlreadyExists(s : string) : boolean;
var i : integer;
begin
  result := false;
  for i := 0 to UnitCount - 1 do
    if Units[i].FileName = s then begin
      result := true;
      break;
    end;
end;

function TProject.Read(name, defaultvalue : string) : string;
begin
  result := IniFile.ReadString('project', name, defaultvalue);
end;

function TProject.ReadBool(name : string; defaultvalue : boolean) : boolean;
begin
 result := IniFile.ReadBool('project', name, defaultvalue);
end;

procedure TProject.Write(name, value : string);
begin
  IniFile.WriteString('project', name, value);
end;

procedure TProject.WriteBool(name : string; value : boolean);
begin
  IniFile.WriteBool('project', name, value);
end;

procedure TProject.OpenUnit(index : integer);
begin
  Units[index].Editor := TEditor.Create(MainForm);
  if Units[index].FileName <> ''then
    try
      Units[index].Editor.Init(true, ExtractFileName(Units[index].Filename), Units[index].Filename, true);
    except
      MessageDlg('Could not open "'+Units[index].Filename+'"', mtWarning, [mbOK], 0);
    end;
end;

procedure TProject.SaveUnitAs(i : integer; sFileName : string);
begin
  {$IFDEF PAS}
  if Units[i].FileName = MainUnit then begin
    MainUnit := sFileName;
    UpdateFile;
  end;
  {$ENDIF}
  Units[i].New := False;
  Units[i].FileName := sFileName;
  Units[i].Node.Text := ExtractFileName(sFileName);
  MainForm.ProjectView.Refresh;
  IniFile.WriteString('unit'+inttostr(i+1), 'filename', sFileName);
  Save;
end;

function TProject.GetUnitFromEditor(ed : TEditor) : integer;
var i : integer;
begin
  result := -1;
  for i := 0 to UnitCount - 1 do
    if Units[i].Editor = ed then
      result := i;
end;

function TProject.GetExecutableName : string;
begin
  if StaticLib then
    result := ChangeFileExt(FileName, LIB_EXT)
  else if DynamicLib then
    result := ChangeFileExt(FileName, SO_EXT)
  else
    result := ChangeFileExt(FileName, '');
end;

end.
